using System;
using System.Configuration;
using System.Windows.Forms;
using Microsoft.DirectX;
using Microsoft.DirectX.Direct3D;
using Microsoft.Samples.DirectX.UtilityToolkit;

namespace Obrancovia
{
    public class HerneJadro : IFrameworkCallback, IDeviceCreation
    {
        #region Vytvorenie
        private Framework sampleFramework = null;
        /// <summary>Vytvor nov instanciu triedy</summary>
        public HerneJadro(Framework f)
        {
            // ulo kontrukciu
            sampleFramework = f;
        }
        #endregion

        #region Kontanty
        public const string NazovHry = "Obrancovia";
        public static readonly string MediaPath = ConfigurationManager.AppSettings.Get("MediaPath");
        private const float ZobrazVyhruPoDobu = 2.5f;

        // Kontanty pre kameru.
        private const float NajvyssiaRychlost = 30.0f;
        private static readonly Vector3 ZakladneUmiestnenieKamery = new Vector3(
          -2.5f, 25.0f, -55.0f);
        private static readonly Vector3 ZakladnySmerPohladuKamery = new Vector3(
          -2.5f, 0.0f, 0.0f);
        private static readonly Vector3 KameraHore = new Vector3(0, 1, 0);

        // Kontanty pre svetl.
        private static readonly Vector3 SvetloVyberuLoopyho = new Vector3(
          0, 0.0f, 1.0f);
        private static readonly Vector3 SvetloUrovne = new Vector3(0, -0.6f, 0.4f);
        #endregion

        #region Premenn triedy
        // Umon ukoni aplikciu ak chcem
        private bool canClose = false;

        // Nahrva sa prve dan rove?
        private bool jeUrovenNahravana = false;

        // as, po ktor je zobrazen obrazovka s oznmenm, e hr vyhral.
        private float trvanieVyhryUrovne = 0.0f;

        // Ktor rove prve hrajem.
        private int poradieUrovne = 1;

        // Je obrazovka na potvdenie konca hry zobrazen?
        private bool jeKoniecZobrazeny = false;

        // Mesh a textra na zobrazenie rovne hry
        private Mesh levelMesh = null;
        private Texture[] levelTextures = null;

        // Objekt triedy hernej postavy.
        private Player postava = null;

        // Aktulna rove.
        private Uroven aktualnaUroven = null;

        // Hlavn tl psma.
        private Font pismoHry = null;

        // Aktulne umiestnenie kamery.
        private Vector3 aktualnaPoziciaKamery;

        // Je obrazovka hlavnho menu zobrazen?
        private bool jeHlavneMenuZobrazene = true;

        // Obrazovka hlavnho menu hry.
        private MainUiScreen obrazovkaHlavnehoMenu = null;

        // Obrazovka s potvrdenm konca hry.
        private QuitScreen obrazovkaKonca = null;

        // Obrazovka na vber hernej postavy.
        private SelectLoopyScreen obrazovkaVyberu = null;

        // Je obrazovka na vber hernej postavy zobrazen?
        private bool jeVyberZobrazeny = false;

        // Textry zdiean obrazovkami uivatelskho rozhrania.
        private Texture texturaTlacitka = null;

        #endregion

        #region Ladiaci vstup
        #if (DEBUG)
        private Font debugFont = null;
        #endif
        #endregion

        /// <summary>
        /// Funkcia sa vol pri uvdzan zariadenia do chodu, kontroluje, i zariadenie 
        /// m vetky potrebn vlastnosti au tch, ktor neprejd
/// Testom, vracia false.
        /// </summary>
        public bool IsDeviceAcceptable(Caps caps, Format adapterFormat, Format backBufferFormat, bool vOkne)
        {
            // Vynechme formty back bufferu, ktor nepodporuj alfa mieanie.
            if (!Manager.CheckDeviceFormat(caps.AdapterOrdinal, caps.DeviceType,
                 adapterFormat, Usage.QueryPostPixelShaderBlending,
                 ResourceType.Textures, backBufferFormat))
                return false;

            // Vynechme zariadenia, ktor nepodporuj osvetlenie.
            if (caps.MaxActiveLights == 0)
                return false;

            return true;
        }

        /// <summary>
        /// Tto funkcia sa vol tesne pred uvedenm zariadenia do chodu a nastavuje
/// rzne nastavenia tohto zariadenia. Medzi podporovan parametre patr aj tie,
/// ktor boli nastaven vzorov kontrukci pre toto zariadenie, a touto
/// funkci ich mete meni. Dvajte si pozor, aby ste parametre nastavili 
        /// sprvne, inak uvedenie zariadenia do chodu zlyh.
        /// </summary>
        public void ModifyDeviceSettings(DeviceSettings nastavenia, Caps caps)
        {
            // Tento program sli pre prcu s benmi zariadeniami, preto pouvajte
// Len ben zariadenia
            if ((caps.DeviceCaps.SupportsPureDevice) &&
                ((nastavenia.BehaviorFlags & CreateFlags.HardwareVertexProcessing)
                != 0))
                nastavenia.BehaviorFlags |= CreateFlags.PureDevice;
        }

         /// <summary>
 /// Tto udalos je vyhoden hne potom, o je Direct3D zariadenie je vytvoren,
 /// a to poas spania programu. Vytvorm Pool.Managed zdroje.
 /// </ Summary>
        private void OnCreateDevice(object sender, DeviceEventArgs e)
        {
            SurfaceDescription desc = e.BackBufferDescription;
            #if (DEBUG)
                // In debug mode, use this font to render stats
                debugFont = new Font(e.Device, new System.Drawing.Font("Arial", 12));
            #endif

            // Vytvorm psmo hry.
            pismoHry = new Font(e.Device, new System.Drawing.Font("Arial", 28.0f));

            // Vytvorm textry pre obrazovky uivateskho rozhrania.
            texturaTlacitka = TextureLoader.FromFile(e.Device, HerneJadro.MediaPath
            + "tlacitka.png");

            // Vytvorm obrazovku na vber hernej postavy.
            obrazovkaVyberu = new SelectLoopyScreen(e.Device, texturaTlacitka,
              desc.Width, desc.Height);
            obrazovkaVyberu.Zvolena += new EventHandler(OnLoopySelected);

            // Vytvorm obrazovku hlavnho menu.
            obrazovkaHlavnehoMenu = new MainUiScreen(e.Device, desc.Width, desc.Height);
            obrazovkaHlavnehoMenu.NovaHra += new EventHandler(OnNewGame);
            obrazovkaHlavnehoMenu.Koniec += new EventHandler(OnMainQuit);

            // Vytvorm instanciu triedy QuitScreen.
            obrazovkaKonca = new QuitScreen(e.Device, texturaTlacitka, desc.Width, desc.Height);
            obrazovkaKonca.Cancel += new EventHandler(OnQuitScreenCancel);
            obrazovkaKonce.QuitGame += new EventHandler(OnQuitScreenQuit);
        }

        /// <summary>
/// Tto udalos je vyhoden hne potom, o je Direct3D zariadenie
/// resetovan, a to po strate scenra zariadenia.
/// Vytvorm Pool.Default.
/// </ Summary>

        private void OnResetDevice(object sender, DeviceEventArgs e)
        {
            SurfaceDescription desc = e.BackBufferDescription;
            // Nastavm transforman maticu.
            e.Device.Transform.Projection = Matrix.PerspectiveFovLH((float)Math.PI / 4, 
                (float)desc.Width / (float)desc.Height, 0.1f, 1000000.0f);

            e.Device.Transform.View = Matrix.LookAtLH(aktualniPoziceKamery, 
                ZakladnySmerPohladuKamery, KameraHore);

            // Nastavm svetlo.

            // Svetlo 0 bude svetlo rovne.
            e.Device.Lights[0].DiffuseColor = new ColorValue(1.0f, 1.0f, 1.0f, 1.0f);
            e.Device.Lights[0].Direction = SvetloUrovne;
            e.Device.Lights[0].Type = LightType.Directional;
            // Svetlo 1 bude svetlo pre obrazovku vberu hernej postavy.
            e.Device.Lights[1].DiffuseColor = new ColorValue(1.0f, 1.0f, 1.0f, 1.0f);
            e.Device.Lights[1].Direction = SvetloVyberuLoopyho;
            e.Device.Lights[1].Type = LightType.Directional;

            PovolSvetla();

            // Ulom schopnosti zariadenia
            Caps caps = sampleFramework.DeviceCaps;

            // Nastavm filtrovanie textr
            if (caps.TextureFilterCaps.SupportsMinifyAnisotropic)
            {
                e.Device.SamplerState[0].MinFilter = TextureFilter.Anisotropic;
            }
            else if (caps.TextureFilterCaps.SupportsMinifyLinear)
            {
                e.Device.SamplerState[0].MinFilter = TextureFilter.Linear;
            }
            if (caps.TextureFilterCaps.SupportsMagnifyAnisotropic)
            {
                e.Device.SamplerState[0].MagFilter = TextureFilter.Anisotropic;
            }
            else if (caps.TextureFilterCaps.SupportsMagnifyLinear)
            {
                e.Device.SamplerState[0].MagFilter = TextureFilter.Linear;
            }
            if (caps.TextureFilterCaps.SupportsMipMapLinear)
            {
                e.Device.SamplerState[0].MipFilter = TextureFilter.Linear;
            }

        #if (DEBUG)
            debugFont.OnResetDevice();
        #endif

            // Zaistme, aby sa objekt typu Font choval sprvne pri retartovn.
            pismoHry.OnResetDevice();
        }

        /// <summary>
/// Tto funkcia je zavolan potom, o Direct3D zariadenie prejde do stavu
/// Straten a predtm, ako je volan funkcia Device.Reset (). zdroje
/// Vytvoren callback funkci OnResetDevice by mali by uvonen tu,
/// Hlavne medzi ne patr Pool.Default zdroje. 
        /// </summary>
  
        private void OnLostDevice(object sender, EventArgs e)
        {
        #if (DEBUG)
            if (debugFont != null)
            {
                debugFont.OnLostDevice();
            }
        #endif

            if (pismoHry != null)
            {
                // Uistm sa, e nasledujc objekt typu Font uprace vytvoren zdroje.
                pismoHry.OnLostDevice();
            }
        }

        /// <summary>
        /// Tto funkcia je zavolan potom, o Direct3D zariadenie prejde do stavu
        /// Straten a predtm, ako je volan funkcia Device.Reset (). zdroje
        /// Vytvoren callback funkci OnResetDevice by mali by uvonen tu,
        /// Hlavne medzi ne patr Pool.Default zdroje. 
        /// </summary>
        private void OnDestroyDevice(object sender, EventArgs e)
        {
        #if (DEBUG)
            if (debugFont != null)
            {
                // klid fontu
                debugFont.Dispose();
                debugFont = null;
            }
        #endif
            // Upracem obrazovky UI.
            if (obrazovkaKonca != null)
            {
                obrazovkaKonca.Dispose();
            }
            if (obrazovkaVyberu != null)
            {
                obrazovkaVyberu.Dispose();
            }
            if (obrazovkaHlavnehoMenu != null)
            {
                obrazovkaHlavnehoMenu.Dispose();
            }
            // Ak je postava null a som na obrazovke na vber hernej postavy,
            // upracem sie hernej postavy.
            if ((postava == null) && (obrazovkaVyberu != null))
            {
                obrazovkaVyberu.UvolniMeshLoopyho();
            }
            // Upraceme hern postavu.
            if (postava != null)
            {
                postava.Dispose();
            }

            // Upraceme textry pre obrazovky uvateskho rozhrania.
            if (texturaTlacitka != null)
            {
                texturaTlacitka.Dispose();
            }

            if (pismoHry != null)
            {
                // Upratovanie objektu typu Font.
                pismoHry.Dispose();
                pismoHry = null;
            }

            // Upratovanie textr pouitch pre mesh rovne
            if ((levelTextures != null) && (levelTextures.Length > 0))
            {
                for (int i = 0; i < levelTextures.Length; i++)
                {
                    levelTextures[i].Dispose();
                }
                // Upratovanie samotnho meshu rovn.
                levelMesh.Dispose();
            }

            // Upraceme kocky.
            Kocka.Cleanup();
        }

        /// <summary>
        /// Tto callback funkcia bude zavolan na zaiatku kadho snmku.
        /// </summary>
        public void OnFrameMove(Device zariadenie, double appCas, float uplynulyCas)
        {
            if (FrameworkTimer.IsStopped)
                return; // Neni o robi

            if ((jeKonecZobrazeny) || (jehlavneMenuZobrazene) ||
              (jeVyberZobrazeny))
            {
                return; // Ni nemusm robi.
            }

            // Aktualizujem hern postavu.
            postava.Aktualizuj(uplynulyCas, (float)appCas);

            // Uistm sa, e hr nezane hra,kn sa kamera nepresunie na
            // zkladn pozciu.
            AktualizujKameru(uplynulyCas);

            // Skontrolujem, i obrazovka s oznmenm, e hr vyhral, m zosta
            // zobrazen.
            if ((aktualnaUroven.JeHraVyhrana) &&
              (trvanieVyhryUrovne > ZobrazVyhruPoDobu))
            {
                // Uistm sa, e alia rove existuje.
                if (Uroven.SkontrolujExistenciuSuboru(poradieUrovne + 1))
                {
                    poradieUrovne++;
                    NahrajPrislusnuUroven();
                }
            }

            // Ak hr vyhral, upravm premenn trvanieVyhryUrovne.
            if (aktualnaUroven.JeHraVyhrana)
            {
                trvanieVyhryUrovne += uplynulyCas;
            }
            // Aktualizujeme dan rove.
            if (!jeUrovenNahravana)
            {
                aktualnaUroven.Aktualizuj(uplynulyCas);
            }
        }

        /// <summary>
        /// Tto callback funkcia bude zavolan na konci kadho snmku, aby
        /// zaistila vetky zobrazenia danej scny a taktie bude zavolan, ak sa
        /// prekresuje okno. Po nvratu z tejto funkcie vzorov kontrukcia vol
        /// zariadenie, aby zobrazilo obsah zobrazovacej pamti.
        /// </summary>
        public void OnFrameRender(Device zariadenie, double appCas, float uplynulyCas)
        {
            bool beginSceneVolano = false;

            // Vyistm zobrazovac target a zbuffer
            zariadenie.Clear(ClearFlags.ZBuffer | ClearFlags.Target, 0, 1.0f, 0);
            try
            {
                zariadenie.BeginScene();
                beginSceneVolano = true;

                // Rozhodnem sa, o zobrazm.
                if (jeHlavneMenuZobrazene)
                {
                    obrazovkaHlavnehoMenu.Draw();
                }
                else if (jeVyberZobrazeny)
                {
                    obrazovkaVyberu.Draw((float)appCas);
                }
                else
                {
                    ZobrazScenuHry(zariadenie, appCas);
                }

                #if (DEBUG)
                // Zobraz ladc informace
                debugFont.DrawText(null, sampleFramework.FrameStats, new System.Drawing.Rectangle(2,0,0,0),
                    DrawTextFormat.NoClip, unchecked((int)0xffffff00));
                debugFont.DrawText(null, sampleFramework.DeviceStats, new System.Drawing.Rectangle(2,15,0,0),
                    DrawTextFormat.NoClip, unchecked((int)0xffffff00));
                #endif
            }
            finally
            {
                if (beginSceneVolano)
                    zariadenie.EndScene();
            }
        }

        #region Metdy vytvorenia
        private void VytvorGrafickePrvky(SelectLoopyScreen obrazovka)
        {
            // Mm nahra mesh rovne?
            if (levelMesh == null)
            {
                // Nahrajem mesh rovne.
                ExtendedMaterial[] mtrls;
                levelMesh = Mesh.FromFile(MediaPath + "uroven.x", MeshFlags.Managed,
                  sampleFramework.Device, out mtrls);

                // Ulom materily na neskr a nahrajem textry.
                if ((mtrls != null) && (mtrls.Length > 0))
                {
                    levelTextures = new Texture[mtrls.Length];
                    for (int i = 0; i < mtrls.Length; i++)
                    {
                        // Nahrajem textru.
                        levelTextures[i] = TextureLoader.FromFile(sampleFramework.Device, MediaPath +
                          mtrls[i].TextureFilename);
                    }
                }
            }

            // Vytvorme objekt hernej postavy.
            postava = new Player(obrazovka.MeshLoopyho, obrazovka.TexturaLoopyho,
            obrazovka.MaterialLoopyho);

            // Nahrajem prslun rove.
            NahrajPrislusnuUroven();
        }

        /// <summary>
        /// Nahraje aktulnu urove.
        /// </summary>
        private void NahrajPrislusnuUroven()
        {
            // Nahrajem nau rove
            aktualnaUroven = new Uroven(poradieUrovne, sampleFramework.Device, postava);

            // Nastavm nhodn pozciu kamery.
            NastavNahodnuPoziciuKamery();

            jeUrovenNahravana = true;
            trvaniVyhryUrovne = 0.0f;
        }
        #endregion

        /// <summary>
        /// Tto metda zobraz scnu rovne.
        /// </summary>
        private void ZobrazScenuHry(Device zariadenie, double appCas)
        {
            // Najprv zobrazme mesh rovne, ale predtm vypneme zbuffer,
            // protoe ho nebudem potebova.
            zarizeni.RenderState.ZBufferEnable = false;
            zarizeni.RenderState.ZBufferWriteEnable = false;

            zarizeni.Transform.World = Matrix.Scaling(15, 15, 15) *
              Matrix.Translation(0, 22, 0);
            zarizeni.RenderState.Lighting = false;
            for (int i = 0; i < levelTextures.Length; i++)
            {
                zariadenie.SetTexture(0, levelTextures[i]);
                levelMesh.DrawSubset(i);
            }

            // Op zapnem zbuffer.
            zariadenie.RenderState.ZBufferEnable = true;
            zariadenie.RenderState.ZBufferWriteEnable = true;

            // Teraz zapnem osvetlenie.
            zarizeni.RenderState.Lighting = true;

            // Zobrazm hern postavu.
            postava.Draw(zariadenie, (float)appCas);

            aktualnaUroven.Draw(zariadenie);

            int width = sampleFramework.DeviceSettings.presentParams.BackBufferWidth;
            int height = sampleFramework.DeviceSettings.presentParams.BackBufferHeight;
        if (jeKoniecZobrazeny)
            {
                obrazovkaKonca.Draw();
            }
            if (jeUrovenNahravana)
            {
                string textNahravani = string.Format(
                  "Nahrvam rove {0}\r\nSte pripraven?", poradieUrovne);
                if (!jeKoniecZobrazeny)
                {
                    pismoHry.DrawText(null, textNahravania, new System.Drawing.Rectangle(
                        System.Drawing.Point.Empty, new System.Drawing.Size(width, height)),
                      DrawTextFormat.Center | DrawTextFormat.VerticalCenter |
                      DrawTextFormat.NoClip, System.Drawing.Color.WhiteSmoke);
                }
            }
            else
            {
                // Ak je rove nahran, zistm stav hry.
                if (aktualnaUroven != null)
                {
                    // Zobrazm aktulny stav.
                    string informacieOKockach = string.Format(
                      "Zostva kociek {0} - Sprvnych {1}",
                      aktualnaUroven.PocetKociek - aktualnaUroven.PocetSpravnychKociek,
                      aktualnaUroven.PocetSpravnychKociek);

                    System.Drawing.Color barvaStatistik = System.Drawing.Color.White;
                    // Zmenm farbu, ak m viac ne polovica kociek sprvnu farbu.
                    if ((aktualnaUroven.PocetKociek - aktualnaUroven.PocetSpravnychKociek)
                      > aktualnaUroven.PocetSpravnychKociek)
                    {
                        farbaStatistik = System.Drawing.Color.Turquoise;
                    }

                    pismoHry.DrawText(null, informacieOKockach,
                      new System.Drawing.Rectangle(0, height - 60,
                      width, height),
                      DrawTextFormat.Center | DrawTextFormat.NoClip,
                      farbaStatistik);

                    if (!aktualnaUroven.JeKoniecHry)
                    {
                        string stavHry = string.Format(
                          "Tahov: {0} (Max: {1})\r\nZostavajuci cas: {2}",
                          aktualnaUroven.CelkomTahov, aktualnaUroven.MaximalneTahov,
                          aktualnaUroven.ZostavajuciCas);

                        pismoHry.DrawText(null, stavHry, new System.Drawing.Rectangle(0, 30,
                          width, height),
                          DrawTextFormat.Center | DrawTextFormat.NoClip,
                          System.Drawing.Color.Yellow);
                    }
                    else
                    {
                        string stavHry;
                        System.Drawing.Color c;
                        if (aktualnaUroven.JeHraVyhrana)
                        {
                            stavHry = string.Format("Gratulujeme! Vyhrali ste tto rove!");
                            c = System.Drawing.Color.White;
                        }
                        else
                        {
                            stavHry = string.Format("Koniec hry!!");
                            c = System.Drawing.Color.Red;
                        }
                        pismoHry.DrawText(null, stavHry, new System.Drawing.Rectangle(0, 30,
                          width, height),
                          DrawTextFormat.Center | DrawTextFormat.NoClip,
                          c);
                    }
                }
            }
        }


        #region Osvetlenie

        /// <summary>
        /// Zapnem prslun svetlo scny.
        /// </summary>
        private void PovolSvetla()
        {
            // Zapnem sprvne svetlo, druh vypnem.
            if (jeVyberZobrazeny)
            {
                sampleFramework.Device.Lights[1].Enabled = true;
                sampleFramework.Device.Lights[0].Enabled = false;
            }
            else
            {
                sampleFramework.Device.Lights[0].Enabled = true;
                sampleFramework.Device.Lights[1].Enabled = false;
            }

        }
        #endregion

        #region Kamera

        /// <summary>
        /// Nastavm nhodn pozciu, na ktor kamera prelet.
        /// </summary>
        private void NastavNahodnuPoziciuKamery()
        {
            Random r = new Random();
            aktualnaPoziciaKamery = new Vector3(
              (float)r.NextDouble() * 5.0f + 15.0f,
              (float)r.NextDouble() * 10.0f + 120.0f,
              (float)r.NextDouble() * 5.0f + 15.0f
              );
            // Nhodn zmena.
            if (r.Next(100) > 49)
            {
                aktualnaPoziciaKamery.X *= -1;
            }
            // Nhodn zmena.
            if (r.Next(100) > 49)
            {
                aktualnaPoziciaKamery.Z *= -1;
            }
        }
        /// <summary>
        /// Aktualizuje pozciu kamery, ak neni na cieovej pozcii.
        /// </summary>
        /// <param name="uplynulyCas">as, ktor uplynul od poslednej zmeny</param>
        private void AktualizujKameru(float uplynulyCas)
        {
            if (aktualnaPoziciaKamery != ZakladneUmiestnenieKamery)
            {
                Vector3 diff = ZakladneUmiestnenieKamery - aktualnaPoziciaKamery;
                // Som dostatene blzko, aby som sa mohol presun okamite?
                if (diff.Length() > (NajvyssiaRychlost * uplynulyCas))
                {
                    // Nie, musm sa pomaly presva
                    diff.Normalize();
                    diff.Scale(NajvyssiaRychlost * uplynulyCas);
                    aktualnaPoziciaKamery += diff;
                }
                else
                {
                    // no, presuniem sa okamite.
                    aktualnaPoziciaKamery = ZakladneUmiestnenieKamery;
                }
                // Nastavme transformciu pohadu.
                sampleFramework.Device.Transform.View = Matrix.LookAtLH(aktualnaPoziciaKamery,
                ZakladnySmerPohladuKamery, KameraHore);
            }
            else
            {
                jeUrovenNahravana = false;
            }
        }
        #endregion

        #region Uvatesk vstup
        /// <summary>Obsluhuje potaovu my.</summary>
        private void OnMouseEvent(Object sender, MouseEventArgs e)
        {
            if (e.Button == MouseButtons.None)
            {
                if (jeHlavneMenuZobrazene)
                {
                    obrazovkaHlavnehoMenu.OnMouseMove(e.X, e.Y);
                }
                else if (jeVyberZobrazeny)
                {
                    obrazovkaVyberu.OnMouseMove(e.X, e.Y);
                }
                else if (jeKoniecZobrazeny)
                {
                    obrazovkaKonca.OnMouseMove(e.X, e.Y);
                }
            }
            if (e.Button == MouseButtons.Left)
            {
                if (jeHlavneMenuZobrazene)
                {
                    obrazovkaHlavnehoMenu.OnMouseClick(e.X, e.Y);
                }
                
                else if (jeVyberZobrazeny)
                {
                    obrazovkaVyberu.OnMouseClick(e.X, e.Y);
                }
                else if (jeKoniecZobrazeny)
                {
                    obrazovkaKonca.OnMouseClick(e.X, e.Y);
                }
            }
        }

        /// <summary>Obsluhuje klvesnicu.</summary>
        private void OnKeyEvent(Object sender, KeyEventArgs e)
        {
            if (jeHlavneMenuZobrazene)
            {
                obrazovkaHlavnehoMenu.OnKeyPress(e.KeyCode);
            }
            else if (jeVyberZobrazeny)
            {
                obrazovkaVyberu.OnKeyPress(e.KeyCode);
            }
            else if (jeKoniecZobrazeny)
            {
                obrazovkaKonca.OnKeyPress(e.KeyCode);
            }
            else
            {
                // Umonm hrovi kedykovek opusti hru.
                if (e.KeyCode == System.Windows.Forms.Keys.Escape)
                {
                    ZobrazObrazovkuKonca();
                }

                // Ignorujem uvatesk vstup, ke kamera prelieta na zkladn pozciu.
                if (!jeUrovenNahravana)
                {
                    // Predme zpracovanie udalost objektu rovne.
                    aktualnaUroven.OnKeyPress(e.KeyCode);
                }
            }            
        }


        /// <summary>
        /// Umon zavrenie hry iba cez ukonovaciu obrazovku
        /// </summary>
        private IntPtr OnWndProc(IntPtr hWnd, NativeMethods.WindowMessage msg, IntPtr wParam, IntPtr lParam, ref bool noFurtherProcessing)
        {
            if (msg == NativeMethods.WindowMessage.Close)
            {
                ZobrazObrazovkuKonca();
                if (!canClose)
                {
                    noFurtherProcessing = true;
                    return Framework.TrueIntPtr;
                }
            }
            return IntPtr.Zero;
        }

        #endregion

        private void ZobrazObrazovkuKonca()
        {
            jeKoniecZobrazeny = !jeKoniecZobrazeny;
            if (jeKoniecZobrazeny)
            {
                // Hra je teraz pozastaven, pozastavm tie asova a zobrazme vzvu
                // k potvrdeniu konca hry.
                sampleFramework.Pause(true, false);
            }
            else
            {
                sampleFramework.Pause(false, false);
            }
        }

        #region Obsluha udalosti obrazovek UI
        /// <summary>
        /// Spusten, ke hr nechce hru opusti.
        /// </summary>
        private void OnQuitScreenCancel(object sender, EventArgs e)
        {
            // Hra je pozastaven, uvate neshlasil s ukonenm hry, pokraujeme
            // v hre.
            sampleFramework.Pause(false, false);
            jeKoniecZobrazeny = false;
        }

        /// <summary>
        /// Spusten, ke hr chce hru ukoni (klikol na potvrdzovacie tlatko).
        /// </summary>
        private void OnQuitScreenQuit(object sender, EventArgs e)
        {
            // Hra je pozastaven, uvate shlasil s ukonenm hry, prejdem na
            // obrazovku s hlavnm menu hry.
            jeHlavneMenuZobrazene = true;
            jeKoniecZobrazeny = false;
            sampleFramework.Pause(false, false);
            // Poradie rovne nastavme sp na 1.
            poradieUrovne = 1;
        }

        /// <summary>
        /// Spusten, ke hr vybral hern postavu.
        /// </summary>
        private void OnLoopySelected(object sender, EventArgs e)
        {
            SelectLoopyScreen obrazovka = sender as SelectLoopyScreen;
            System.Diagnostics.Debug.Assert(obrazovka != null, NazovHry,
              "Obrazovka pre vyber hernej postavy by nemala byt null.");

            // Vytvorm grafick prvky.
            VytvorGrafickePrvky(obrazovka);

            // A schovm obrazovku na vber hernej postavy.
            jeVyberZobrazeny = false;

            // Nastavme osvetlenie.
            PovolSvetla();
        }

        #endregion

        #region Obsluha udalosti obrazovky hlavneho menu
        private void OnNewGame(object sender, EventArgs e)
        {
            // Zobrazm obrazovku na vber hernej postavy.
            jeHlavneMenuZobrazene = false;
            jeVyberZobrazeny = true;

            // Zapnem svetlo.
            PovolSvetla();
        }

        private void OnMainQuit(object sender, EventArgs e)
        {
            // Teraz umonm ukonenie aplikcie
            canClose = true;
            sampleFramework.CloseWindow();
        }
        #endregion

        #region Metoda Main
        /// <summary>
        /// Vstupn bod do programu. Zana program a spa cyklick proces
        /// spracovania sprv. Behom neinnosti (ke sa nezpracuvvaj sprvy)
        /// sa zobrazuje scna.
        /// </summary>
        static int Main() 
        {
            using(Framework sampleFramework = new Framework())
            {
                HerneJadro KockaEngine = new HerneJadro(sampleFramework);
                 // Nastavm callback funkcie. Tieto funkcie umonia vzorov kontrukci, aby
 // upozornila program na zmeny zariadenia, uvatesk vstupy a sprvy
 // systmu Windows. Callback funkcie s nepovinn, preto nastavme len
 // callback funkcie pre udalosti, ktor ns zaujmaj. Ak ale nenastavte
 // callback funkcie pre reset alebo stratu zariadenia, potom vzorov kontrukcia
 // neme resetova zariadenia, km program neuvon vetky zdroje tohto
 // zariadenie. Podobne musme nastavi callback funkcie pre vytvorenie / ruenie / / zariadenie.

                sampleFramework.Disposing += new EventHandler(KockaEngine.OnDestroyDevice);
                sampleFramework.DeviceLost += new EventHandler(KockaEngine.OnLostDevice);
                sampleFramework.DeviceCreated +=
                 new DeviceEventHandler(KockaEngine.OnCreateDevice);
                sampleFramework.DeviceReset +=
                 new DeviceEventHandler(KockaEngine.OnResetDevice);

                // Zachytvanie udalost myou.
                sampleFramework.IsNotifiedOnMouseMove = true;
                sampleFramework.SetCallbackInterface(KockaEngine);

                try
                {
                  #if (!DEBUG)
                  // V ostrej verzi printim hru, aby sa zobrazila cez cel obrazovku.
                  sampleFramework.IsOverridingFullScreen = true;
                  #endif
                  // Nastavme kurzor.
                  sampleFramework.SetCursorSettings(true, true);

                  // Nastavm zaiaton hodnoty vzorov kontrukcie a vytvorm poadovan
                  // okno a Direct3D zariadenie pre hru. iadn z tchto funkc vola
                  // nemusme, ale umon nm nastavi chovnie vzorovej kontrukcie.
                  sampleFramework.Initialize( false, false, true );
                  sampleFramework.CreateWindow("Obrancovia  Hra");

                  sampleFramework.Window.KeyDown += new KeyEventHandler(KockaEngine.OnKeyEvent);
                  sampleFramework.Window.MouseMove += new MouseEventHandler(KockaEngine.OnMouseEvent);
                  sampleFramework.Window.MouseClick += new MouseEventHandler(KockaEngine.OnMouseEvent);

                  sampleFramework.CreateDevice( 0, true, Framework.DefaultSizeWidth,
                    Framework.DefaultSizeHeight, KockaEngine);

                  // Prenechm obsluhu sprv a zobrazovania na vzorovej kontrukcii. Vzorov
                  // kontrukcia bude vola callback funkcie FrameMove a FrameRender, ke
                  // prve nebude obsluhova iadnu sprvu.
                  sampleFramework.MainLoop();
                }
                #if(DEBUG)
                catch (Exception e)
                {
                  sampleFramework.DisplayErrorMessage(e);
                #else
                catch
                {
                    // V ostrej verzi chybov sprvu nezobrazm.                
                #endif
                    // Ignorujem vnimky, mali by by oetren inde.
                    return (sampleFramework.ExitCode == 0) ? 1 : sampleFramework.ExitCode;
                }

                // Tu mete vykona vetky upratovacie prce.
                // ...
                // Direct3D objekty s uvonen automaticky cez ich callback metdy.
                return sampleFramework.ExitCode;
            }
        }
        #endregion
    }
}
