using System;
using System.Configuration;
using System.Windows.Forms;
using Microsoft.DirectX;
using Microsoft.DirectX.Direct3D;
using Microsoft.Samples.DirectX.UtilityToolkit;

namespace Obrancovia
{
    public class HerneJadro : IFrameworkCallback, IDeviceCreation
    {
        #region Vytvorenie
        private Framework sampleFramework = null;
        public HerneJadro(Framework f)
        {
          sampleFramework = f;
        }
        #endregion

        #region Konstanty
        public const string NazovHry = "Obrancovia";
        public static readonly string MediaPath = ConfigurationManager.AppSettings.Get("MediaPath");
        private const float ZobrazVyhruPoDobu = 2.5f;

        private const float NajvyssiaRychlost = 30.0f;
        private static readonly Vector3 ZakladneUmiestnenieKamery = new Vector3(
          -2.5f, 25.0f, -55.0f);
        private static readonly Vector3 ZakladnySmerPohladuKamery = new Vector3(
          -2.5f, 0.0f, 0.0f);
        private static readonly Vector3 KameraHore = new Vector3(0, 1, 0);

        private static readonly Vector3 SvetloVyberuLoopyho = new Vector3(
          0, 0.0f, 1.0f);
        private static readonly Vector3 SvetloUrovne = new Vector3(0, -0.6f, 0.4f);
        #endregion

        #region Premenne tridy
        private bool canClose = false;

        private bool jeUrovenNahravana = false;

        private float trvanieVyhryUrovne = 0.0f;

        private int poradieUrovne = 1;

        private bool jeKoniecZobrazeny = false;

        private Mesh levelMesh = null;
        private Texture[] levelTextures = null;

        private Player postava = null;

        private Uroven aktualnaUroven = null;

        private Font pismoHry = null;

        private Vector3 aktualnaPoziciaKamery;

        private bool jeHlavneMenuZobrazene = true;

        private MainUiScreen obrazovkaHlavnehoMenu = null;

        private QuitScreen obrazovkaKonca = null;

        private SelectLoopyScreen obrazovkaVyberu = null;

        private bool jeVyberZobrazeny = false;

        private Texture texturaTlacitka = null;

        #endregion

        #region Ladici vystup
        #if (DEBUG)
        private Font debugFont = null;
        #endif
        #endregion


        public bool IsDeviceAcceptable(Caps caps, Format adapterFormat, Format backBufferFormat, bool vOkne)
        {
            if (!Manager.CheckDeviceFormat(caps.AdapterOrdinal, caps.DeviceType,
                 adapterFormat, Usage.QueryPostPixelShaderBlending,
                 ResourceType.Textures, backBufferFormat))
                return false;

            if (caps.MaxActiveLights == 0)
                return false;

            return true;
        }

        public void ModifyDeviceSettings(DeviceSettings nastavenia, Caps caps)
        {
            if ((caps.DeviceCaps.SupportsPureDevice) &&
                ((nastavenia.BehaviorFlags & CreateFlags.HardwareVertexProcessing)
                != 0))
                nastavenia.BehaviorFlags |= CreateFlags.PureDevice;
        }

        private void OnCreateDevice(object sender, DeviceEventArgs e)
        {
            SurfaceDescription desc = e.BackBufferDescription;
            #if (DEBUG)
                // In debug mode, use this font to render stats
                debugFont = new Font(e.Device, new System.Drawing.Font("Arial", 12));
            #endif

            pismoHry = new Font(e.Device, new System.Drawing.Font("Arial", 28.0f));

            texturaTlacitka = TextureLoader.FromFile(e.Device, HerneJadro.MediaPath
            + "tlacitka.png");

            obrazovkaVyberu = new SelectLoopyScreen(e.Device, texturaTlacitka,
              desc.Width, desc.Height);
            obrazovkaVyberu.Zvolena += new EventHandler(OnLoopySelected);

            obrazovkaHlavnehoMenu = new MainUiScreen(e.Device, desc.Width, desc.Height);
            obrazovkaHlavnehoMenu.NovaHra += new EventHandler(OnNewGame);
            obrazovkaHlavnehoMenu.Koniec += new EventHandler(OnMainQuit);

            obrazovkaKonca = new QuitScreen(e.Device, texturaTlacitka, desc.Width, desc.Height);
            obrazovkaKonca.Cancel += new EventHandler(OnQuitScreenCancel);
            obrazovkaKonce.QuitGame += new EventHandler(OnQuitScreenQuit);
        }

        private void OnResetDevice(object sender, DeviceEventArgs e)
        {
            SurfaceDescription desc = e.BackBufferDescription;
            e.Device.Transform.Projection = Matrix.PerspectiveFovLH((float)Math.PI / 4, 
                (float)desc.Width / (float)desc.Height, 0.1f, 1000000.0f);

            e.Device.Transform.View = Matrix.LookAtLH(aktualniPoziceKamery, 
                ZakladnySmerPohladuKamery, KameraHore);


            e.Device.Lights[0].DiffuseColor = new ColorValue(1.0f, 1.0f, 1.0f, 1.0f);
            e.Device.Lights[0].Direction = SvetloUrovne;
            e.Device.Lights[0].Type = LightType.Directional;
            e.Device.Lights[1].DiffuseColor = new ColorValue(1.0f, 1.0f, 1.0f, 1.0f);
            e.Device.Lights[1].Direction = SvetloVyberuLoopyho;
            e.Device.Lights[1].Type = LightType.Directional;

            PovolSvetla();

            Caps caps = sampleFramework.DeviceCaps;

            if (caps.TextureFilterCaps.SupportsMinifyAnisotropic)
            {
                e.Device.SamplerState[0].MinFilter = TextureFilter.Anisotropic;
            }
            else if (caps.TextureFilterCaps.SupportsMinifyLinear)
            {
                e.Device.SamplerState[0].MinFilter = TextureFilter.Linear;
            }
            if (caps.TextureFilterCaps.SupportsMagnifyAnisotropic)
            {
                e.Device.SamplerState[0].MagFilter = TextureFilter.Anisotropic;
            }
            else if (caps.TextureFilterCaps.SupportsMagnifyLinear)
            {
                e.Device.SamplerState[0].MagFilter = TextureFilter.Linear;
            }
            if (caps.TextureFilterCaps.SupportsMipMapLinear)
            {
                e.Device.SamplerState[0].MipFilter = TextureFilter.Linear;
            }

        #if (DEBUG)
            debugFont.OnResetDevice();
        #endif

            pismoHry.OnResetDevice();
        }

        private void OnLostDevice(object sender, EventArgs e)
        {
        #if (DEBUG)
            if (debugFont != null)
            {
                debugFont.OnLostDevice();
            }
        #endif

            if (pismoHry != null)
            {
                pismoHry.OnLostDevice();
            }
        }

        private void OnDestroyDevice(object sender, EventArgs e)
        {
        #if (DEBUG)
            if (debugFont != null)
            {
                // klid fontu
                debugFont.Dispose();
                debugFont = null;
            }
        #endif
            if (obrazovkaKonca != null)
            {
                obrazovkaKonca.Dispose();
            }
            if (obrazovkaVyberu != null)
            {
                obrazovkaVyberu.Dispose();
            }
            if (obrazovkaHlavnehoMenu != null)
            {
                obrazovkaHlavnehoMenu.Dispose();
            }
            if ((postava == null) && (obrazovkaVyberu != null))
            {
                obrazovkaVyberu.UvolniMeshLoopyho();
            }
            if (postava != null)
            {
                postava.Dispose();
            }

            if (texturaTlacitka != null)
            {
                texturaTlacitka.Dispose();
            }

            if (pismoHry != null)
            {
                pismoHry.Dispose();
                pismoHry = null;
            }

            if ((levelTextures != null) && (levelTextures.Length > 0))
            {
                for (int i = 0; i < levelTextures.Length; i++)
                {
                    levelTextures[i].Dispose();
                }
                levelMesh.Dispose();
            }

            Kocka.Cleanup();
        }


        public void OnFrameMove(Device zariadenie, double appCas, float uplynulyCas)
        {
            if (FrameworkTimer.IsStopped)
                return;

            if ((jeKonecZobrazeny) || (jehlavneMenuZobrazene) ||
              (jeVyberZobrazeny))
            {
                return;
            }

            postava.Aktualizuj(uplynulyCas, (float)appCas);

            AktualizujKameru(uplynulyCas);

            if ((aktualnaUroven.JeHraVyhrana) &&
              (trvanieVyhryUrovne > ZobrazVyhruPoDobu))
            {
                if (Uroven.SkontrolujExistenciuSuboru(poradieUrovne + 1))
                {
                    poradieUrovne++;
                    NahrajPrislusnuUroven();
                }
            }

            if (aktualnaUroven.JeHraVyhrana)
            {
                trvanieVyhryUrovne += uplynulyCas;
            }
            if (!jeUrovenNahravana)
            {
                aktualnaUroven.Aktualizuj(uplynulyCas);
            }
        }

        public void OnFrameRender(Device zariadenie, double appCas, float uplynulyCas)
        {
            bool beginSceneVolano = false;

            zariadenie.Clear(ClearFlags.ZBuffer | ClearFlags.Target, 0, 1.0f, 0);
            try
            {
                zariadenie.BeginScene();
                beginSceneVolano = true;

                if (jeHlavneMenuZobrazene)
                {
                    obrazovkaHlavnehoMenu.Draw();
                }
                else if (jeVyberZobrazeny)
                {
                    obrazovkaVyberu.Draw((float)appCas);
                }
                else
                {
                    ZobrazScenuHry(zariadenie, appCas);
                }

                #if (DEBUG)
                // Zobraz ladc informace
                debugFont.DrawText(null, sampleFramework.FrameStats, new System.Drawing.Rectangle(2,0,0,0),
                    DrawTextFormat.NoClip, unchecked((int)0xffffff00));
                debugFont.DrawText(null, sampleFramework.DeviceStats, new System.Drawing.Rectangle(2,15,0,0),
                    DrawTextFormat.NoClip, unchecked((int)0xffffff00));
                #endif
            }
            finally
            {
                if (beginSceneVolano)
                    zariadenie.EndScene();
            }
        }

        #region Metody vytvorenia
        private void VytvorGrafickePrvky(SelectLoopyScreen obrazovka)
        {
            if (levelMesh == null)
            {
                ExtendedMaterial[] mtrls;
                levelMesh = Mesh.FromFile(MediaPath + "uroven.x", MeshFlags.Managed,
                  sampleFramework.Device, out mtrls);

                if ((mtrls != null) && (mtrls.Length > 0))
                {
                    levelTextures = new Texture[mtrls.Length];
                    for (int i = 0; i < mtrls.Length; i++)
                    {
                        levelTextures[i] = TextureLoader.FromFile(sampleFramework.Device, MediaPath +
                          mtrls[i].TextureFilename);
                    }
                }
            }

            postava = new Player(obrazovka.MeshLoopyho, obrazovka.TexturaLoopyho,
            obrazovka.MaterialLoopyho);

            NahrajPrislusnuUroven();
        }

        private void NahrajPrislusnuUroven()
        {
            aktualnaUroven = new Uroven(poradieUrovne, sampleFramework.Device, postava);

            NastavNahodnuPoziciuKamery();

            jeUrovenNahravana = true;
            trvaniVyhryUrovne = 0.0f;
        }
        #endregion

        private void ZobrazScenuHry(Device zariadenie, double appCas)
        {
            zarizeni.RenderState.ZBufferEnable = false;
            zarizeni.RenderState.ZBufferWriteEnable = false;

            zarizeni.Transform.World = Matrix.Scaling(15, 15, 15) *
              Matrix.Translation(0, 22, 0);
            zarizeni.RenderState.Lighting = false;
            for (int i = 0; i < levelTextures.Length; i++)
            {
                zariadenie.SetTexture(0, levelTextures[i]);
                levelMesh.DrawSubset(i);
            }

            zariadenie.RenderState.ZBufferEnable = true;
            zariadenie.RenderState.ZBufferWriteEnable = true;

            zarizeni.RenderState.Lighting = true;

            postava.Draw(zariadenie, (float)appCas);

            aktualnaUroven.Draw(zariadenie);

            int width = sampleFramework.DeviceSettings.presentParams.BackBufferWidth;
            int height = sampleFramework.DeviceSettings.presentParams.BackBufferHeight;

            if (jeKoniecZobrazeny)
            {
                obrazovkaKonca.Draw();
            }
            if (jeUrovenNahravana)
            {
                string textNahravania = string.Format(
                  "Nahravam uroven {0}\r\nSi pripraveny?", poradieUrovne);
                if (!jeKoniecZobrazeny)
                {
                    pismoHry.DrawText(null, textNahravania, new System.Drawing.Rectangle(
                        System.Drawing.Point.Empty, new System.Drawing.Size(width, height)),
                      DrawTextFormat.Center | DrawTextFormat.VerticalCenter |
                      DrawTextFormat.NoClip, System.Drawing.Color.WhiteSmoke);
                }
            }
            else
            {
                if (aktualnaUroven != null)
                {
                    string informacieOKockach = string.Format(
                      "Zostava kociek {0} - Spravnych {1}",
                      aktualnaUroven.PocetKociek - aktualnaUroven.PocetSpravnychKociek,
                      aktualnaUroven.PocetSpravnychKociek);

                    System.Drawing.Color farbaStatistik = System.Drawing.Color.White;
                    if ((aktualnaUroven.PocetKociek - aktualnaUroven.PocetSpravnychKociek)
                      > aktualnaUroven.PocetSpravnychKociek)
                    {
                        farbaStatistik = System.Drawing.Color.Turquoise;
                    }

                    pismoHry.DrawText(null, informacieOKockach,
                      new System.Drawing.Rectangle(0, height - 60,
                      width, height),
                      DrawTextFormat.Center | DrawTextFormat.NoClip,
                      farbaStatistik);

                    if (!aktualnaUroven.JeKoniecHry)
                    {
                        string stavHry = string.Format(
                          "Tahov: {0} (Maximum: {1})\r\nZostavajuci cas: {2}",
                          aktualnaUroven.CelkomTahov, aktualnaUroven.MaximalneTahov,
                          aktualnaUroven.ZostavajuciCas);

                        pismoHry.DrawText(null, stavHry, new System.Drawing.Rectangle(0, 30,
                          width, height),
                          DrawTextFormat.Center | DrawTextFormat.NoClip,
                          System.Drawing.Color.Yellow);
                    }
                    else
                    {
                        string stavHry;
                        System.Drawing.Color c;
                        if (aktualnaUroven.JeHraVyhrana)
                        {
                            stavHry = string.Format("Gratulujeme! Vyhral si tuto uroven!");
                            c = System.Drawing.Color.White;
                        }
                        else
                        {
                            stavHry = string.Format("Koniec hry!!");
                            c = System.Drawing.Color.Red;
                        }
                        pismoHry.DrawText(null, stavHry, new System.Drawing.Rectangle(0, 30,
                          width, height),
                          DrawTextFormat.Center | DrawTextFormat.NoClip,
                          c);
                    }
                }
            }
        }


        #region Osvetlenie

        private void PovolSvetla()
        {
            if (jeVyberZobrazeny)
            {
                sampleFramework.Device.Lights[1].Enabled = true;
                sampleFramework.Device.Lights[0].Enabled = false;
            }
            else
            {
                sampleFramework.Device.Lights[0].Enabled = true;
                sampleFramework.Device.Lights[1].Enabled = false;
            }

        }
        #endregion

        #region Kamera

        private void NastavNahodnuPoziciuKamery()
        {
            Random r = new Random();
            aktualnaPoziciaKamery = new Vector3(
              (float)r.NextDouble() * 5.0f + 15.0f,
              (float)r.NextDouble() * 10.0f + 120.0f,
              (float)r.NextDouble() * 5.0f + 15.0f
              );
            if (r.Next(100) > 49)
            {
                aktualnaPoziciaKamery.X *= -1;
            }
            if (r.Next(100) > 49)
            {
                aktualnaPoziciaKamery.Z *= -1;
            }
        }
        /// <param name="uplynulyCas">as, ktor uplynul od poslednej zmeny</param>
        private void AktualizujKameru(float uplynulyCas)
        {
            if (aktualnaPoziciaKamery != ZakladneUmiestnenieKamery)
            {
                Vector3 diff = ZakladneUmiestnenieKamery - aktualnaPoziciaKamery;
                if (diff.Length() > (NajvyssiaRychlost * uplynulyCas))
                {
                    diff.Normalize();
                    diff.Scale(NajvyssiaRychlost * uplynulyCas);
                    aktualnaPoziciaKamery += diff;
                }
                else
                {
                    aktualnaPoziciaKamery = ZakladneUmiestnenieKamery;
                }
                sampleFramework.Device.Transform.View = Matrix.LookAtLH(aktualnaPoziciaKamery,
                ZakladnySmerPohladuKamery, KameraHore);
            }
            else
            {
                jeUrovenNahravana = false;
            }
        }
        #endregion

        #region Uzivatelsky vstup
        /// <summary>Obsluhuje potaovu my.</summary>
        private void OnMouseEvent(Object sender, MouseEventArgs e)
        {
            if (e.Button == MouseButtons.None)
            {
                if (jeHlavneMenuZobrazene)
                {
                    obrazovkaHlavnehoMenu.OnMouseMove(e.X, e.Y);
                }
                else if (jeVyberZobrazeny)
                {
                    obrazovkaVyberu.OnMouseMove(e.X, e.Y);
                }
                else if (jeKoniecZobrazeny)
                {
                    obrazovkaKonca.OnMouseMove(e.X, e.Y);
                }
            }
            if (e.Button == MouseButtons.Left)
            {
                if (jeHlavneMenuZobrazene)
                {
                    obrazovkaHlavnehoMenu.OnMouseClick(e.X, e.Y);
                }
                else if (jeVyberZobrazeny)
                {
                    obrazovkaVyberu.OnMouseClick(e.X, e.Y);
                }
                else if (jeKoniecZobrazeny)
                {
                    obrazovkaKonca.OnMouseClick(e.X, e.Y);
                }
            }
        }

        /// <summary>Obsluhuje klvesnicu.</summary>
        private void OnKeyEvent(Object sender, KeyEventArgs e)
        {
            if (jeHlavneMenuZobrazene)
            {
                obrazovkaHlavnehoMenu.OnKeyPress(e.KeyCode);
            }
            else if (jeVyberZobrazeny)
            {
                obrazovkaVyberu.OnKeyPress(e.KeyCode);
            }
            else if (jeKoniecZobrazeny)
            {
                obrazovkaKonca.OnKeyPress(e.KeyCode);
            }
            else
            {
                if (e.KeyCode == System.Windows.Forms.Keys.Escape)
                {
                    ZobrazObrazovkuKonca();
                }

                if (!jeUrovenNahravana)
                {
                    aktualnaUroven.OnKeyPress(e.KeyCode);
                }
            }            
        }       
        

        private IntPtr OnWndProc(IntPtr hWnd, NativeMethods.WindowMessage msg, IntPtr wParam, IntPtr lParam, ref bool noFurtherProcessing)
        {
            if (msg == NativeMethods.WindowMessage.Close)
            {
                ZobrazObrazovkuKonca();
                if (!canClose)
                {
                    noFurtherProcessing = true;
                    return Framework.TrueIntPtr;
                }
            }
            return IntPtr.Zero;
        }

        #endregion

        private void ZobrazObrazovkuKonca()
        {
            jeKoniecZobrazeny = !jeKoniecZobrazeny;
            if (jeKoniecZobrazeny)
            {
                sampleFramework.Pause(true, false);
            }
            else
            {
                sampleFramework.Pause(false, false);
            }
        }

        #region Obsluha udalosti obrazovek UI
        private void OnQuitScreenCancel(object sender, EventArgs e)
        {
            sampleFramework.Pause(false, false);
            jeKoniecZobrazeny = false;
        }

        private void OnQuitScreenQuit(object sender, EventArgs e)
        {
            jeHlavneMenuZobrazene = true;
            jeKoniecZobrazeny = false;
            sampleFramework.Pause(false, false);
            poradieUrovne = 1;
        }

        private void OnLoopySelected(object sender, EventArgs e)
        {
            SelectLoopyScreen obrazovka = sender as SelectLoopyScreen;
            System.Diagnostics.Debug.Assert(obrazovka != null, NazovHry,
              "Obrazovka pre vyber hernej postavy by nemala byt null.");

            VytvorGrafickePrvky(obrazovka);

            jeVyberZobrazeny = false;

            PovolSvetla();
        }

        #endregion

        #region Obsluha udalosti obrazovky hlavneho menu
        private void OnNewGame(object sender, EventArgs e)
        {
            jeHlavneMenuZobrazene = false;
            jeVyberZobrazeny = true;

            PovolSvetla();
        }

        private void OnMainQuit(object sender, EventArgs e)
        {
            canClose = true;
            sampleFramework.CloseWindow();
        }
        #endregion

        #region Metoda Main
        static int Main() 
        {
            using(Framework sampleFramework = new Framework())
            {
                HerneJadro KockaEngine = new HerneJadro(sampleFramework);

                sampleFramework.Disposing += new EventHandler(KockaEngine.OnDestroyDevice);
                sampleFramework.DeviceLost += new EventHandler(KockaEngine.OnLostDevice);
                sampleFramework.DeviceCreated +=
                 new DeviceEventHandler(KockaEngine.OnCreateDevice);
                sampleFramework.DeviceReset +=
                 new DeviceEventHandler(KockaEngine.OnResetDevice);

                sampleFramework.IsNotifiedOnMouseMove = true;
                sampleFramework.SetCallbackInterface(KockaEngine);

                try
                {
                  #if (!DEBUG)
                  sampleFramework.IsOverridingFullScreen = true;
                  #endif
                  sampleFramework.SetCursorSettings(true, true);

                  sampleFramework.Initialize( false, false, true );
                  sampleFramework.CreateWindow("Obrancovia  Hra");

                  sampleFramework.Window.KeyDown += new KeyEventHandler(KockaEngine.OnKeyEvent);
                  sampleFramework.Window.MouseMove += new MouseEventHandler(KockaEngine.OnMouseEvent);
                  sampleFramework.Window.MouseClick += new MouseEventHandler(KockaEngine.OnMouseEvent);

                  sampleFramework.CreateDevice( 0, true, Framework.DefaultSizeWidth,
                    Framework.DefaultSizeHeight, KockaEngine);

                  sampleFramework.MainLoop();
                }
                #if(DEBUG)
                catch (Exception e)
                {
                  sampleFramework.DisplayErrorMessage(e);
                #else
                catch
                {
                #endif
                  return (sampleFramework.ExitCode == 0) ? 1 : sampleFramework.ExitCode;
                }

                return sampleFramework.ExitCode;
            }
        }
        #endregion
    }
}
